package com.njyunzhi.pet_identity.service.impl;

import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyV3Result;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.njyunzhi.pet_identity.common.Constants;
import com.njyunzhi.pet_identity.common.StringUtils;
import com.njyunzhi.pet_identity.entity.TaApplication;
import com.njyunzhi.pet_identity.entity.TaOrder;
import com.njyunzhi.pet_identity.mapper.TaApplicationMapper;
import com.njyunzhi.pet_identity.mapper.TaOrderMapper;
import com.njyunzhi.pet_identity.service.ITaOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * <p>
 * 订单表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-05-24
 */
@Service
public class TaOrderServiceImpl extends BaseServiceImpl<TaOrderMapper, TaOrder> implements ITaOrderService {

    @Autowired
    TaApplicationMapper applicationMapper;

    @Override
    public void notifyOrderPay(WxPayOrderNotifyV3Result.DecryptNotifyResult result) throws Exception {
        TaOrder taOrder = getExistBy("order_no", result.getOutTradeNo(), false, true);
        if (null == taOrder) {
            throw new Exception("未找到单号 [" + result.getOutTradeNo() + "] 对应的订单");
        }

        if (taOrder.getStatus() == Constants.PAY_STATUS_PAID) {
            return;
        }

        boolean isSuccess = WxPayConstants.ResultCode.SUCCESS.equals(result.getTradeState());
        if (!isSuccess) return;

        taOrder.setStatus(Constants.PAY_STATUS_PAID);
        taOrder.setPayDate(LocalDateTime.now());

        baseMapper.updateById(taOrder);

        // 更新申请单状态
        if (StringUtils.isEmpty(taOrder.getApplyId())) return;
        TaApplication taApplication = applicationMapper.selectById(taOrder.getApplyId());
        if (null == taApplication || Constants.STATUS_DELETE == taApplication.getStatus()) return;

        taApplication.setPayStatus(Constants.PAY_STATUS_PAID);
        applicationMapper.updateById(taApplication);
    }

    @Override
    public String createOrderNo() throws Exception {
        return baseMapper.getShortUUID();
    }
}
